@@ -198,6 +198,7 @@ cvar_t cl_cmdline			= {"cl_cmdline", "", CVAR_ROM};
 cvar_t cl_useproxy			= {"cl_useproxy", "0"};
 cvar_t cl_proxyaddr         = {"cl_proxyaddr", ""};
 cvar_t cl_window_caption	= {"cl_window_caption", "1"};
+cvar_t cl_window_caption_delimiter = {"cl_window_caption_delimiter", " | "};
 
 cvar_t cl_model_bobbing		= {"cl_model_bobbing", "1"};
 cvar_t cl_nolerp			= {"cl_nolerp", "0"}; // 0 is good for indep-phys, 1 is good for old-phys
@@ -421,6 +422,10 @@ void CL_MakeActive(void)
 	if (!cls.demoseeking) {
 		Con_ClearNotify ();
 	}
+	
+	// Reset safestrafe state on spawn
+	memset(&cl.safestrafe, 0, sizeof(cl.safestrafe));
+	
 	TP_ExecTrigger("f_spawn");
 }
 
@@ -808,6 +813,7 @@ void CL_Connect_f (void)
 	qbool proxy;
 	char *connect_addr = NULL;
 	char *server_buf = NULL;
+	portpingprobe_status_t status;
 
 	if (Cmd_Argc() != 2) 
 	{
@@ -815,6 +821,24 @@ void CL_Connect_f (void)
 		return;
 	}
 
+	// It's not possible to change the source port when connected to a
+	// server. Terminate the current connection before proceeding.
+	if (IsPortPingProbeEnabled() && cls.state != ca_disconnected)
+	{
+		Host_EndGame();
+	}
+
+	// The user has already initiated a port ping probe, so we'll abort that
+	// and wait for the worker thread to stop before we proceed.
+	if (IsPortPingProbeEnabled() && (status = NET_GetPortPingProbeStatus()) == PORTPINGPROBE_PROBING)
+	{
+		NET_SetPortPingProbeStatus(PORTPINGPROBE_ABORT);
+		while ((status = NET_GetPortPingProbeStatus()) != PORTPINGPROBE_READY)
+		{
+			Sys_MSleep(10);
+		}
+	}
+
 	// in this part proxy means QWFWD proxy
 	if (cl_proxyaddr.string[0]) {
 		char *secondproxy;
@@ -849,6 +873,19 @@ void CL_Connect_f (void)
 		connected_via_proxy = false;
 	}
 
+	if (IsPortPingProbeEnabled())
+	{
+		if (status == PORTPINGPROBE_READY)
+		{
+			NET_PortPingProbe(connect_addr, Cmd_Argv(1));
+			return;
+		}
+		else if (status == PORTPINGPROBE_COMPLETED)
+		{
+			NET_SetPortPingProbeStatus(PORTPINGPROBE_READY);
+		}
+	}
+
 	// in this part proxy means Qizmo proxy
 	proxy = cl_useproxy.value && CL_ConnectedToProxy();
 
@@ -1340,9 +1377,20 @@ void CL_Disconnect (void)
 
 void CL_Disconnect_f (void) 
 {
+	portpingprobe_status_t status;
+
 	cl.intermission = 0;
 	CL_Demo_Disconnected();
 
+	if (IsPortPingProbeEnabled() && (status = NET_GetPortPingProbeStatus()) == PORTPINGPROBE_PROBING)
+	{
+		NET_SetPortPingProbeStatus(PORTPINGPROBE_ABORT);
+		while ((status = NET_GetPortPingProbeStatus()) != PORTPINGPROBE_READY)
+		{
+			Sys_MSleep(10);
+		}
+	}
+
 	Host_EndGame();
 }
 
@@ -1390,6 +1438,9 @@ void CL_Reconnect_f (void)
 		// switch the stuff
 		Cbuf_AddText(va("connect %s\n", cls.servername));
 	}
+	else if (IsPortPingProbeEnabled()) {
+		Cbuf_AddText(va("connect %s\n", cls.servername));
+	}
 	else {
 		// good old reconnect, no need to do anything special
 		Host_EndGame();
@@ -1761,6 +1812,7 @@ static void CL_InitLocal(void)
 	Cvar_Register(&cl_shownet);
 	Cvar_Register(&cl_confirmquit);
 	Cvar_Register(&cl_window_caption);
+	Cvar_Register(&cl_window_caption_delimiter);
 	Cvar_Register(&cl_onload);
 
 #ifdef WIN32
@@ -2767,6 +2819,18 @@ void CL_UpdateCaption(qbool force)
 	else if (cl_window_caption.integer == 2) {
 		snprintf(str, sizeof(str), "ezQuake");
 	}
+	else if (cl_window_caption.integer == 3) {
+		if (cls.state < ca_connected) {
+			snprintf(str, sizeof(str), "ezQuake v%s", VERSION_NUMBER);
+		}
+		else {
+			snprintf(str, sizeof(str), "%d/%d%s%s",
+				TP_CountPlayers(),
+				Q_atoi(Info_ValueForKey(cl.serverinfo, "maxclients")),
+				cl_window_caption_delimiter.string,
+				TP_MapName());
+		}
+	}
 
 	if (force || strcmp(str, caption)) {
 		VID_SetCaption(str);
