@@ -773,6 +773,14 @@ typedef struct {
 	float map_fog_density;
 	qbool map_fog_enabled;
 	float map_fog_sky;
+
+	// sv_safestrafe client-side prediction
+	struct {
+		int      pending_frames;    // Frames of forced stop remaining
+		float    pending_direction; // Desired sidemove after stop
+		int      stop_frames;       // Accumulated frames with sidemove=0
+		float    last_sidemove;     // Previous frame's sidemove value
+	} safestrafe;
 } clientState_t;
 
 #define SCORING_SYSTEM_DEFAULT   0
