@@ -40,6 +40,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "stats_grid.h"
 #include "tp_triggers.h"
 #include "fs.h"
+#include "rulesets.h"
 
 typedef struct commandline_option_s {
 	const char* name;
@@ -264,7 +265,7 @@ void COM_FileBase (const char *in, char *out)
 //
 // If path doesn't have a .EXT, append extension (extension should include the .)
 //
-void COM_DefaultExtension (char *path, char *extension)
+void COM_DefaultExtension (char *path, char *extension, size_t path_size)
 {
 	char *src;
 
@@ -276,7 +277,7 @@ void COM_DefaultExtension (char *path, char *extension)
 		src--;
 	}
 
-	strlcat (path, extension, MAX_OSPATH);
+	strlcat (path, extension, path_size);
 }
 
 // If path doesn't have an extension or has a different extension, append(!) specified extension
@@ -1630,6 +1631,11 @@ void Com_PrintVerticalBar(int width)
 
 void COM_ParseIPCData(const char *buf, unsigned int bufsize)
 {
+	if (Rulesets_RestrictIPC()) {
+		Com_Printf("The use of IPC is not allowed during matches\n");
+		return;
+	}
+
 	if (bufsize > 0)
 	{
 		// TODO : Expect some more fancy commands and stuff here instead.. if we want to use it for more than qw:// urls...
