@@ -206,6 +206,10 @@ static r_program_uniform_t program_uniforms[] = {
 	{ r_program_post_process_glc, "v_blend", 1, false },
 	// r_program_uniform_post_process_glc_contrast,
 	{ r_program_post_process_glc, "contrast", 1, false },
+	// r_program_uniform_post_process_glc_r_inv_width,
+	{ r_program_post_process_glc, "r_inv_width", 1, false },
+	// r_program_uniform_post_process_glc_r_inv_height,
+	{ r_program_post_process_glc, "r_inv_height", 1, false },
 	// r_program_uniform_sky_glc_cameraPosition,
 	{ r_program_sky_glc, "cameraPosition", 1, false },
 	// r_program_uniform_sky_glc_speedscale,
@@ -214,6 +218,8 @@ static r_program_uniform_t program_uniforms[] = {
 	{ r_program_sky_glc, "skySpeedscale2", 1, false },
 	// r_program_uniform_sky_glc_skyTex,
 	{ r_program_sky_glc, "skyTex", 1, false },
+	// r_program_uniform_sky_glc_skyWind,
+	{ r_program_sky_glc, "skyWind", 1, false },
 	// r_program_uniform_sky_glc_skyDomeTex,
 	{ r_program_sky_glc, "skyDomeTex", 1, false },
 	// r_program_uniform_sky_glc_skyDomeCloudTex,
@@ -330,6 +336,8 @@ static r_program_uniform_t program_uniforms[] = {
 	{ r_program_fx_world_geometry, "outline_color", 1, false },
 	// r_program_uniform_outline_depth_threshold
 	{ r_program_fx_world_geometry, "outline_depth_threshold", 1, false },
+	// r_program_uniform_outline_normal_threshold
+	{ r_program_fx_world_geometry, "outline_normal_threshold", 1, false },
 	// r_program_uniform_outline_scale
 	{ r_program_fx_world_geometry, "outline_scale", 1, false },
 	// r_program_uniform_aliasmodel_outline_color_model
@@ -580,8 +588,12 @@ static const char* safe_strstr(const char* source, size_t max_length, const char
 	size_t search_length = strlen(search_string);
 	const char* position;
 
-	position = (const char*)memchr(source, search_string[0], max_length);
-	while (position) {
+	while (true) {
+		position = (const char*)memchr(source, search_string[0], max_length);
+		if (!position) {
+			break;
+		}
+
 		// Move along
 		if (max_length < (position - source)) {
 			break;
@@ -596,13 +608,14 @@ static const char* safe_strstr(const char* source, size_t max_length, const char
 		}
 
 		// Try again
-		source = position;
-		position = (const char*)memchr(source + 1, search_string[0], max_length);
+		source = position + 1;
+		max_length -= 1;
 	}
 
 	return NULL;
 }
 
+#if 0
 static void GL_DebugPrintShaderText(GLuint shader)
 {
 	int length = 0;
@@ -615,6 +628,7 @@ static void GL_DebugPrintShaderText(GLuint shader)
 	DebugOutput(src);
 	Q_free(src);
 }
+#endif
 
 static int GL_InsertDefinitions(
 	const char* strings[],
@@ -685,7 +699,6 @@ static qbool GL_CompileProgram(
 	GLint result = 0;
 	int i;
 
-	const char* friendlyName = program->friendly_name;
 	GLsizei vertex_components = 1;
 	const char* vertex_shader_text[MAX_SHADER_COMPONENTS] = { program->shaders[shadertype_vertex].text, "", "", "", "", "" };
 	GLint vertex_shader_text_length[MAX_SHADER_COMPONENTS] = { program->shaders[shadertype_vertex].length, 0, 0, 0, 0, 0 };
@@ -696,7 +709,7 @@ static qbool GL_CompileProgram(
 	const char* fragment_shader_text[MAX_SHADER_COMPONENTS] = { program->shaders[shadertype_fragment].text, "", "", "", "", "" };
 	GLint fragment_shader_text_length[MAX_SHADER_COMPONENTS] = { program->shaders[shadertype_fragment].length, 0, 0, 0, 0, 0 };
 
-	DebugOutput(va("Compiling: %s\n", friendlyName));
+	DebugOutput(va("Compiling: %s\n", program->friendly_name));
 
 	R_ProgramBuildStandardDefines(program->standard_option_flags);
 
