@@ -38,7 +38,12 @@ void GLM_StateBeginAliasModelZPassBatch(void);
 void GLM_StateBeginAliasModelBatch(qbool translucent, qbool additive);
 void GLM_StateBeginAliasOutlineBatch(void);
 
-#define MAXIMUM_ALIASMODEL_DRAWCALLS MAX_STANDARD_ENTITIES   // ridiculous
+// MAX_STANDARD_ENTITIES used to be 512, so lets pretend like
+// there can't be more aliasmodel entities, as that will cause
+// the arrays sized according to this limit to become too big
+// for caches etc. Should there be more aliasmodels than this,
+// there will be an error printed.
+#define MAXIMUM_ALIASMODEL_DRAWCALLS 512   // ridiculous
 #define MAXIMUM_MATERIAL_SAMPLERS 32
 
 typedef enum aliasmodel_draw_type_s {
@@ -291,6 +296,7 @@ static void GLM_QueueAliasModelDrawImpl(
 	aliasmodel_draw_instructions_t* instr;
 	int textureSampler = -1;
 	extern cvar_t gl_spec_xray;
+	int i;
 
 	// Compile here so we can work out how many samplers we have free to allocate per draw-call
 	if (!GLM_CompileAliasModelProgram()) {
@@ -364,8 +370,8 @@ static void GLM_QueueAliasModelDrawImpl(
 		int bc = 16 * (bound(0, ent->scoreboard->bottomcolor, 13)) + 8;
 		byte top[] = { host_basepal[tc * 3], host_basepal[tc * 3 + 1], host_basepal[tc * 3 + 2] };
 		byte bot[] = { host_basepal[bc * 3], host_basepal[bc * 3 + 1], host_basepal[bc * 3 + 2] };
-		for(int i = 0; i < 3; i++) uniform->plrtopcolor[i] = (float)top[i] / 256.0f;
-		for(int i = 0; i < 3; i++) uniform->plrbotcolor[i] = (float)bot[i] / 256.0f;
+		for(i = 0; i < 3; i++) uniform->plrtopcolor[i] = (float)top[i] / 256.0f;
+		for(i = 0; i < 3; i++) uniform->plrbotcolor[i] = (float)bot[i] / 256.0f;
 	}
 
 	// Add to queues
@@ -501,7 +507,8 @@ static void GLM_RenderPreparedEntities(aliasmodel_draw_type_t type)
 		renderer.TextureUnitBind(TEXTURE_UNIT_MATERIAL, shelltexture);
 	}
 
-	if (translucent && !shells) {
+	// Depth pre-pass to make viewmodel look good.
+	if (type == aliasmodel_draw_postscene) {
 		GLM_StateBeginAliasModelZPassBatch();
 		for (i = 0; i < instr->num_calls; ++i) {
 			GL_MultiDrawArraysIndirect(
