@@ -61,7 +61,7 @@ qbool Rulesets_RestrictSetEval(void);
 qbool Rulesets_RestrictSetEx(void);
 qbool Rulesets_AllowNoShadows(void);
 qbool Rulesets_RestrictTCL(void);
-qbool Rulesets_RestrictSound(const char* name);
+qbool Rulesets_RestrictPlay(const char* name);
 int Rulesets_MaxSequentialWaitCommands(void);
 qbool Ruleset_BlockHudPicChange(void);
 qbool Ruleset_AllowPolygonOffset(entity_t* ent);
