@@ -144,6 +144,11 @@ cvar_t  scr_scoreboard_wipeout	 	  = {"scr_scoreboard_wipeout",   	  "1"};
 cvar_t	scr_scoreboard_classic        = {"scr_scoreboard_classic", "0"};
 cvar_t	scr_scoreboard_highlightself  = {"scr_scoreboard_highlightself", "1"};
 cvar_t	scr_scoreboard_showclock      = {"scr_scoreboard_showclock", "0"};
+cvar_t	scr_scoreboard_showmapname    = {"scr_scoreboard_showmapname", "0"};
+
+static void OnChange_scr_scoreboard_showqtvusers(cvar_t *old_value, char *new_value, qbool*);
+cvar_t scr_scoreboard_showqtvusers = {"scr_scoreboard_showqtvusers", "1", 0, OnChange_scr_scoreboard_showqtvusers};
+cvar_t scr_scoreboard_qtv_name = {"scr_scoreboard_qtv_name", "\xF1\xF4\xF6"};
 
 // VFrags: only draw the frags for the first player when using mvinset
 #define MULTIVIEWTHISPOV() ((!cl_multiview.value) || (cl_mvinset.value && CL_MultiviewCurrentView() == 1))
@@ -336,6 +341,9 @@ void Sbar_Init(void)
 	Cvar_Register(&scr_scoreboard_classic);
 	Cvar_Register(&scr_scoreboard_highlightself);
 	Cvar_Register(&scr_scoreboard_showclock);
+	Cvar_Register(&scr_scoreboard_showmapname);
+	Cvar_Register(&scr_scoreboard_showqtvusers);
+	Cvar_Register(&scr_scoreboard_qtv_name);
 
 	Cvar_ResetCurrentGroup();
 
@@ -441,9 +449,7 @@ void Sbar_DrawNum (int x, int y, int num, int digits, int color) {
 
 // this used to be static function
 int	Sbar_ColorForMap (int m) {
-	m = bound(0, m, 13);
-
-	return 16 * m + 8;
+	return m == 16 ? 0 : 16 * bound(0, m, 15) + 8;
 }
 
 // HUD -> hexum
@@ -1219,6 +1225,8 @@ void Sbar_SoloScoreboard (void)
 {
 	char	str[256];
 	int	len;
+	int	clock_y;
+	int	mapname_y;
 
 	if (cl.gametype == GAME_COOP)
 	{
@@ -1238,10 +1246,50 @@ void Sbar_SoloScoreboard (void)
 		len = strlen (str);
 		Sbar_DrawString (160 - len*4, 4, str);
 	}
-	else if (scr_scoreboard_showclock.value)
+	else
 	{
-		strlcpy(str, SCR_GetTimeString(TIMETYPE_CLOCK, "%H:%M:%S"), sizeof(str));
-		Sbar_DrawString(160 - (strlen(str)*4), -10, str);
+		if ((scr_scoreboard_showclock.integer && !scr_scoreboard_showmapname.integer))
+		{
+			clock_y = -10;
+		}
+		else if (!scr_scoreboard_showclock.integer && scr_scoreboard_showmapname.integer)
+		{
+			mapname_y = -10;
+		}
+		else if (scr_scoreboard_showclock.integer && scr_scoreboard_showmapname.integer == 3)
+		{
+			clock_y = 2;
+			mapname_y = -10;
+		}
+		else
+		{
+			clock_y = -10;
+			mapname_y = 2;
+		}
+
+		if (scr_scoreboard_showclock.integer)
+		{
+			snprintf(str, sizeof(str), "%s", SCR_GetTimeString(TIMETYPE_CLOCK, "%H:%M:%S"));
+			Sbar_DrawString(160 - strlen(str) * 4, clock_y, str);
+		}
+
+		if (scr_scoreboard_showmapname.integer)
+		{
+			switch (scr_scoreboard_showmapname.integer)
+			{
+				case 2:
+					snprintf(str, sizeof(str), "%s", cl.levelname);
+					break;
+				case 3:
+					snprintf(str, sizeof(str), "%s", host_mapname.string);
+					break;
+				default:
+					snprintf(str, sizeof(str), "%s (%s)", cl.levelname, host_mapname.string);
+					break;
+			}
+
+			Sbar_DrawString (160 - strlen(str) * 4, mapname_y, str);
+		}
 	}
 }
 
@@ -1269,6 +1317,8 @@ static qbool Sbar_ShowTeamKills(void)
 	}
 }
 
+extern qtvuser_t *qtvuserlist;
+
 static void Sbar_DeathmatchOverlay(int start)
 {
 	int stats_team, stats_touches, stats_caps, playerstats[7];
@@ -1287,8 +1337,9 @@ static void Sbar_DeathmatchOverlay(int start)
 	qbool proportional = scr_scoreboard_proportional.integer;
 	qbool any_flags = false;
 	extern ti_player_t ti_clients[MAX_CLIENTS];
-	qbool is_classic = scr_scoreboard_classic.value;
+	int is_classic = scr_scoreboard_classic.integer;
 	qbool is_classic_spec;
+	qtvuser_t *qu;
 
 	if (!start && hud_faderankings.value) {
 		Draw_FadeScreen(hud_faderankings.value);
@@ -1403,24 +1454,24 @@ static void Sbar_DeathmatchOverlay(int start)
 	x += FONT_WIDTH;
 	Draw_SStringAligned(x, y - 8, "ping", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 4);
 	if (is_classic)
-		Draw_SStringAligned(x, y, "\x1d\x1e\x1e\x1f", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 4);
+		Draw_SStringAligned(x, y, is_classic == 2 ? "----" : "\x1d\x1e\x1e\x1f", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 4);
 	x += 5 * FONT_WIDTH;
 	Draw_SStringAligned(x, y - 8, "pl", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 2);
 	if (is_classic)
-		Draw_SStringAligned(x, y, "\x1d\x1f", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 2);
+		Draw_SStringAligned(x, y, is_classic == 2 ? "--" : "\x1d\x1f", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 2);
 	x += 3 * FONT_WIDTH;
 	Draw_SStringAligned(x, y - 8, "time", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 4);
 	if (is_classic)
-		Draw_SStringAligned(x, y, "\x1d\x1e\x1e\x1f", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 4);
+		Draw_SStringAligned(x, y, is_classic == 2 ? "----" : "\x1d\x1e\x1e\x1f", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 4);
 	x += 5 * FONT_WIDTH;
 	Draw_SStringAligned(x, y - 8, "frags", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 5);
 	if (is_classic)
-		Draw_SStringAligned(x, y, "\x1d\x1e\x1e\x1e\x1f", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 5);
+		Draw_SStringAligned(x, y, is_classic == 2 ? "-----" : "\x1d\x1e\x1e\x1e\x1f", scale, alpha, proportional, text_align_right, x + FONT_WIDTH * 5);
 	x += 6 * FONT_WIDTH;
 	if (cl.teamplay) {
 		Draw_SStringAligned(x, y - 8, "team", scale, alpha, proportional, text_align_center, x + FONT_WIDTH * 4);
 		if (is_classic)
-			Draw_SStringAligned(x, y, "\x1d\x1e\x1e\x1f", scale, alpha, proportional, text_align_center, x + FONT_WIDTH * 4);
+			Draw_SStringAligned(x, y, is_classic == 2 ? "----" : "\x1d\x1e\x1e\x1f", scale, alpha, proportional, text_align_center, x + FONT_WIDTH * 4);
 		x += 5 * FONT_WIDTH;
 	}
 
@@ -1431,7 +1482,7 @@ static void Sbar_DeathmatchOverlay(int start)
 	Draw_SStringAligned(x, y - 8, "name", scale, alpha, proportional, text_align_left, x + FONT_WIDTH * 15);
 
 	if (is_classic)
-		Draw_SStringAligned(x, y, "\x1d\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1f", scale, alpha, proportional, text_align_left, x + FONT_WIDTH * 15);
+		Draw_SStringAligned(x, y, is_classic == 2 ? "---------------" : "\x1d\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1f", scale, alpha, proportional, text_align_left, x + FONT_WIDTH * 15);
 	x += (any_flags ? 15 : 16) * FONT_WIDTH;
 	if (!is_classic && statswidth) {
 		stats_xoffset = x;
@@ -1820,6 +1871,93 @@ static void Sbar_DeathmatchOverlay(int start)
 		x = startx;
 	}
 
+	if (!scr_scoreboard_showqtvusers.value) {
+		goto finalize;
+	}
+
+	for (qu = qtvuserlist; qu && y <= SCOREBOARD_LASTROW; qu = qu->next) {
+		color_t background;
+		float bk_alpha;
+		byte c = 2;
+		clrinfo_t color;
+		ca_alpha = 1.0f;
+
+		if (!qu->name[0]) {
+			continue;
+		}
+
+		if (is_classic) {
+			bk_alpha = 0;
+		} else if (k == mynum && scr_scoreboard_highlightself.value) {
+			bk_alpha = 1.7 * SCOREBOARD_ALPHA;
+			bk_alpha = min(alpha, 0.75);
+		} else {
+			bk_alpha = SCOREBOARD_ALPHA;
+		}
+
+		background = RGBA_TO_COLOR(host_basepal[c * 3], host_basepal[c * 3 + 1], host_basepal[c * 3 + 2], (byte)(bk_alpha * 255));
+		Draw_AlphaFillRGB(xofs, y, rank_width, skip, background);
+
+		if (!scr_scoreboard_borderless.value) {
+			Draw_Fill(xofs - 1, y, 1, skip, 0);
+			Draw_Fill(xofs - 1 + rank_width + 1, y, 1, skip, 0);
+		}
+
+		snprintf(num, sizeof(num), "%i", 0);
+
+		color.c = is_classic
+			? RGBA_TO_COLOR(0xFF, 0xFF, 0xFF, (byte)(ca_alpha * 255))
+			: RGBA_TO_COLOR(0xAA, 0xAA, 0xDD, (byte)(ca_alpha * 255));
+		color.i = 0;
+		Draw_SColoredStringAligned(x, y, num, &color, 1, scale, alpha * ca_alpha, proportional, text_align_right, x + FONT_WIDTH * 4);
+		x += 4 * FONT_WIDTH;
+
+		if (is_classic) {
+			color.c = RGBA_TO_COLOR(0xFF, 0xFF, 0xFF, (byte)(ca_alpha * 255));
+		} else {
+			if (p == 0) {
+				// 0 - white
+				color.c = RGBA_TO_COLOR(0xFF, 0xFF, 0xFF, (byte)(ca_alpha * 255));
+			} else if (p < 3) {
+				// 1-2 - yellow
+				color.c = RGBA_TO_COLOR(0xCC, 0xDD, 0xDD, (byte)(ca_alpha * 255));
+			} else if (p < 6) {
+				// 3-5 orange
+				color.c = RGBA_TO_COLOR(0xFF, 0x55, 0x00, (byte)(ca_alpha * 255));
+			} else {
+				// 6+ - red
+				color.c = RGBA_TO_COLOR(0xFF, 0x00, 0x00, (byte)(ca_alpha * 255));
+			}
+		}
+
+		if (!is_classic) {
+			Draw_SColoredStringAligned(x, y, num, &color, 1, scale, alpha * ca_alpha, proportional, text_align_right, x + 3 * FONT_WIDTH);
+		}
+
+		x += 4 * FONT_WIDTH;
+
+		color.c = RGBA_TO_COLOR(255, 255, 255, 255);
+
+		if (!is_classic) {
+			snprintf(myminutes, sizeof(myminutes), "%i", 0);
+			Draw_SColoredStringAligned(x, y, myminutes, &color, 1, scale, alpha, proportional, text_align_right, x + 4 * FONT_WIDTH);
+		}
+
+		x += 5 * FONT_WIDTH;
+
+		if (cl.teamplay) {
+			Draw_SStringAligned(is_classic ? x - (8 * FONT_WIDTH) : x, y, is_classic ? "(qtv)" : scr_scoreboard_qtv_name.string, scale, alpha, proportional, text_align_left, x + 10 * FONT_WIDTH);
+		} else {
+			Draw_SStringAligned(is_classic ? x - (8 * FONT_WIDTH): x, y, is_classic ? "(qtv)" : scr_scoreboard_qtv_name.string, scale, alpha, proportional, text_align_left, x + SHORT_SPECTATOR_NAME_LEN * FONT_WIDTH);
+		}
+
+		x += (cl.teamplay ? 11 : 6) * FONT_WIDTH;
+		Draw_SStringAligned(x, y, qu->name, scale, ca_alpha, proportional, text_align_left, x + FONT_WIDTH * 15);
+		y += skip;
+		x = startx;
+	}
+
+finalize:
 	if (!scr_scoreboard_borderless.value) {
 		Draw_Fill(xofs - 1, y - 1, rank_width + 2, 1, 0); //Border - Bottom
 	}
@@ -1833,7 +1971,7 @@ static void Sbar_TeamOverlay(void)
 	mpic_t *pic;
 	qbool proportional = scr_scoreboard_proportional.integer;
 	float lhs;
-	qbool is_classic = scr_scoreboard_classic.value;
+	int is_classic = scr_scoreboard_classic.integer;
 
 	if (key_dest == key_console && !SCR_TakingAutoScreenshot())
 		return;
@@ -1900,33 +2038,33 @@ static void Sbar_TeamOverlay(void)
 
 	Draw_SStringAligned(x, y, "low", 1, 1, proportional, text_align_right, x + 3 * FONT_WIDTH);
 	if (is_classic)
-		Draw_SStringAligned(x, y + 8, "\x1d\x1e\x1e", 1, 1, proportional, text_align_right, x + 3 * FONT_WIDTH);
+		Draw_SStringAligned(x, y + 8, is_classic == 2 ? "---" : "\x1d\x1e\x1e", 1, 1, proportional, text_align_right, x + 3 * FONT_WIDTH);
 	x += 3 * FONT_WIDTH;
 	Draw_SStringAligned(x, y, "/", 1, 1, proportional, text_align_right, x + FONT_WIDTH);
 	if (is_classic)
-		Draw_SStringAligned(x, y + 8, "\x1e", 1, 1, proportional, text_align_right, x + FONT_WIDTH);
+		Draw_SStringAligned(x, y + 8, is_classic == 2 ? "-" : "\x1e", 1, 1, proportional, text_align_right, x + FONT_WIDTH);
 	x += FONT_WIDTH;
 	Draw_SStringAligned(x, y, "avg", 1, 1, proportional, text_align_right, x + FONT_WIDTH * 3);
 	if (is_classic)
-		Draw_SStringAligned(x, y + 8, "\x1e\x1e\x1e", 1, 1, proportional, text_align_right, x + FONT_WIDTH * 3);
+		Draw_SStringAligned(x, y + 8, is_classic == 2 ? "---" : "\x1e\x1e\x1e", 1, 1, proportional, text_align_right, x + FONT_WIDTH * 3);
 	x += 3 * FONT_WIDTH;
 	Draw_SStringAligned(x, y, "/", 1, 1, proportional, text_align_right, x + FONT_WIDTH);
 	if (is_classic)
-		Draw_SStringAligned(x, y + 8, "\x1e", 1, 1, proportional, text_align_right, x + FONT_WIDTH);
+		Draw_SStringAligned(x, y + 8, is_classic == 2 ? "-" : "\x1e", 1, 1, proportional, text_align_right, x + FONT_WIDTH);
 	x += FONT_WIDTH;
 	Draw_SStringAligned(x, y, "high", 1, 1, proportional, text_align_right, x + FONT_WIDTH * 4);
 	if (is_classic)
-		Draw_SStringAligned(x, y + 8, "\x1e\x1e\x1e\x1f", 1, 1, proportional, text_align_right, x + FONT_WIDTH * 4);
+		Draw_SStringAligned(x, y + 8, is_classic == 2 ? "----" : "\x1e\x1e\x1e\x1f", 1, 1, proportional, text_align_right, x + FONT_WIDTH * 4);
 	x += 4 * FONT_WIDTH;
 	x += FONT_WIDTH;
 	Draw_SStringAligned(x, y, "team", 1, 1, proportional, text_align_center, x + FONT_WIDTH * 4);
 	if (is_classic)
-		Draw_SStringAligned(x, y + 8, "\x1d\x1e\x1e\x1f", 1, 1, proportional, text_align_center, x + FONT_WIDTH * 4);
+		Draw_SStringAligned(x, y + 8, is_classic == 2 ? "----" : "\x1d\x1e\x1e\x1f", 1, 1, proportional, text_align_center, x + FONT_WIDTH * 4);
 	x += 4 * FONT_WIDTH;
 	x += FONT_WIDTH;
 	Draw_SStringAligned(x, y, (cl.scoring_system == SCORING_SYSTEM_TEAMFRAGS ? "score" : "total"), 1, 1, proportional, text_align_right, x + FONT_WIDTH * 5);
 	if (is_classic)
-		Draw_SStringAligned(x, y + 8, "\x1d\x1e\x1e\x1e\x1f", 1, 1, proportional, text_align_right, x + FONT_WIDTH * 5);
+		Draw_SStringAligned(x, y + 8, is_classic == 2 ? "-----" : "\x1d\x1e\x1e\x1e\x1f", 1, 1, proportional, text_align_right, x + FONT_WIDTH * 5);
 	x += 5 * FONT_WIDTH;
 	x += FONT_WIDTH;
 	if ((cl.teamfortress || scr_scoreboard_showflagstats.value) && Stats_IsFlagsParsed() && !is_classic) {
@@ -1937,7 +2075,7 @@ static void Sbar_TeamOverlay(void)
 
 	Draw_SStringAligned(x, y, "players", 1, 1, proportional, text_align_left, x + FONT_WIDTH * 7);
 	if (is_classic)
-		Draw_SStringAligned(x, y + 8, "\x1d\x1e\x1e\x1e\x1e\x1e\x1f", 1, 1, proportional, text_align_left, x + FONT_WIDTH * 7);
+		Draw_SStringAligned(x, y + 8, is_classic == 2 ? "-------" : "\x1d\x1e\x1e\x1e\x1e\x1e\x1f", 1, 1, proportional, text_align_left, x + FONT_WIDTH * 7);
 	x = lhs;
 
 	// If scr_scoreboard_classic is true, we've added an extra line for the
@@ -2493,6 +2631,23 @@ static void OnChange_scr_scoreboard_login_flagfile(cvar_t* cv, char* newvalue, q
 	*cancel = CL_LoginImageLoad(newvalue);
 }
 
+static void OnChange_scr_scoreboard_showqtvusers(cvar_t *old_value, char *new_value, qbool *cancel)
+{
+	qbool is_set = (new_value && *new_value == '1');
+	Info_SetValueForKey (cls.userinfo, "qul", is_set ? "1" : "0", MAX_INFO_STRING);
+
+	if (cls.state >= ca_connected)
+	{
+		MSG_WriteByte(&cls.netchan.message, clc_stringcmd);
+		SZ_Print(&cls.netchan.message, va("setinfo \"qul\" \"%s\"", is_set ? "1" : "0"));
+	}
+
+	if (!is_set)
+	{
+		QTV_FreeUserList();
+	}
+}
+
 static mpic_t* CL_LoginFlag(int id)
 {
 	if (id < 0 || id >= login_image_data.image_count) {
