@@ -102,8 +102,8 @@ void Sys_Quit(void)
 void Sys_Init(void)
 {
 #ifdef __APPLE__
-	extern void init_url_handler( void );
-	init_url_handler();
+	extern void init_macos_extras(void);
+	init_macos_extras();
 #endif
 	Cvar_Register(&sys_highpriority);
 	Cvar_Register(&sys_fontsdir);
@@ -125,6 +125,7 @@ void Sys_Error(char *error, ...)
 		fprintf(qconsole_log, "Error: %s\n", string);
 
 	Host_Shutdown ();
+	SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR, "Error", string, NULL);
 	exit(1);
 }
 
@@ -171,7 +172,7 @@ Sys_listdir
 
 dir_t Sys_listdir (const char *path, const char *ext, int sort_type)
 {
-	static file_t list[MAX_DIRFILES];
+	static file_t list[MAX_DIRFILES]; // FIXME: thread un-safe.
 	dir_t dir;
 	char pathname[MAX_OSPATH];
 	DIR *d;
@@ -214,21 +215,15 @@ dir_t Sys_listdir (const char *path, const char *ext, int sort_type)
 		if (!all)
 		{
 			match_data = pcre2_match_data_create_from_pattern(preg, NULL);
-			switch (error = pcre2_match(preg, (PCRE2_SPTR)oneentry->d_name,
-			                      strlen(oneentry->d_name), 0, 0, match_data, NULL))
-			{
-			case 0:
-				pcre2_match_data_free(match_data);
-				break;
-			case PCRE2_ERROR_NOMATCH:
-				pcre2_match_data_free(match_data);
+			error = pcre2_match(preg, (PCRE2_SPTR)oneentry->d_name,
+				strlen(oneentry->d_name), 0, 0, match_data, NULL);
+			pcre2_match_data_free(match_data);
+			if (error < 0) {
+				if (error != PCRE2_ERROR_NOMATCH) {
+					Con_Printf("Sys_listdir: pcre2_match(%s, %s) error code: %d\n",
+						ext, oneentry->d_name, error);
+				}
 				continue;
-			default:
-				Con_Printf("Sys_listdir: pcre2_match(%s, %s) error code: %d\n",
-				           ext, oneentry->d_name, error);
-				pcre2_match_data_free(match_data);
-				pcre2_code_free(preg);
-				return dir;
 			}
 		}
 		snprintf(pathname, sizeof(pathname), "%s/%s", path, oneentry->d_name);
@@ -485,15 +480,15 @@ char *Sys_fullpath(char *absPath, const char *relPath, int maxLength)
 {
     // too small buffer, copy in tmp[] and then look is enough space in output buffer aka absPath
     if (maxLength-1 < PATH_MAX)	{
-			 char tmp[PATH_MAX+1];
-			 if (realpath(relPath, tmp) && absPath && strlen(tmp) < maxLength+1) {
-					strlcpy(absPath, tmp, maxLength+1);
-          return absPath;
-			 }
-
-       return NULL;
+		char tmp[PATH_MAX+1];
+		if (realpath(relPath, tmp) && absPath && strlen(tmp) < maxLength) {
+			strlcpy(absPath, tmp, maxLength);
+			return absPath;
 		}
 
+		return NULL;
+	}
+
     return realpath(relPath, absPath);
 }
 // kazik <--
@@ -687,13 +682,16 @@ int Sys_Script (const char *path, const char *args)
 
 DL_t Sys_DLOpen(const char *path)
 {
-	return dlopen(path,
+	DL_t ret = dlopen(path,
 #ifdef __OpenBSD__
 	              DL_LAZY
 #else
 	              RTLD_NOW
 #endif
 	             );
+	if (!ret)
+		Con_DPrintf("Sys_DLOpen: %s\n", dlerror());
+	return ret;
 }
 
 qbool Sys_DLClose(DL_t dl)
@@ -794,7 +792,7 @@ void Sys_RegisterQWURLProtocol_f(void)
             "Categories=quakeworld;qtv;Game;\n"
             "Exec=%s +qwurl %s\n"
             "Path=%s\n"
-            "MimeType=x-scheme-handler/qw\n"
+            "MimeType=x-scheme-handler/qw;\n"
             "Terminal=false\n"
             "StartupNotify=false\n"
             , exe_path, "%u", com_basedir);
@@ -836,7 +834,6 @@ void Sys_RegisterQWURLProtocol_f(void)
 int Sys_SetPriority(int priority)
 {
 	int ret, i=0;
-	int which = PRIO_PROCESS;
 
 	switch (priority) {
 		case 1:
@@ -864,7 +861,7 @@ void OnChange_sys_highpriority (cvar_t *var, char *s, qbool *cancel)
 	//do not attempt to change priority if cvar is unset
 	if (s[0] == '\0') return;
 
-	int ok, priority, ret;
+	int priority, ret;
 	char *desc;
 
 	priority = Q_atoi(s);
